@@ -1,5 +1,4 @@
-# Copyright 2015, 2016 Rackspace Inc.
-# All Rights Reserved.
+# Copyright 2015 Rackspace
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,79 +12,53 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import os
 import time
-
-from neutron_lib import constants as n_constants
-from oslo_log import log as logging
-from tempest.api.network import base
-from tempest import config
-from tempest.lib.common.utils import test_utils
-from tempest.lib import exceptions
-
-from neutron_lbaas._i18n import _
+from neutron.i18n import _, _LI
 from neutron_lbaas.tests.tempest.v2.clients import health_monitors_client
 from neutron_lbaas.tests.tempest.v2.clients import listeners_client
 from neutron_lbaas.tests.tempest.v2.clients import load_balancers_client
 from neutron_lbaas.tests.tempest.v2.clients import members_client
 from neutron_lbaas.tests.tempest.v2.clients import pools_client
 
+from tempest.api.network import base
+from tempest import clients as tempest_clients
+from tempest import config
+from tempest import exceptions
+from tempest.openstack.common import log as logging
+
 CONF = config.CONF
 
-LOG = logging.getLogger(__name__)
-
-
-def _setup_client_args(auth_provider):
-    """Set up ServiceClient arguments using config settings. """
-    service = CONF.network.catalog_type or 'network'
-    region = CONF.network.region or 'regionOne'
-    endpoint_type = CONF.network.endpoint_type
-    build_interval = CONF.network.build_interval
-    build_timeout = CONF.network.build_timeout
-
-    # The disable_ssl appears in identity
-    disable_ssl_certificate_validation = (
-        CONF.identity.disable_ssl_certificate_validation)
-    ca_certs = None
-
-    # Trace in debug section
-    trace_requests = CONF.debug.trace_requests
-
-    return [auth_provider, service, region, endpoint_type,
-            build_interval, build_timeout,
-            disable_ssl_certificate_validation, ca_certs,
-            trace_requests]
+# Use local tempest conf if one is available.
+# This usually means we're running tests outside of devstack
+if os.path.exists('./tests/tempest/etc/dev_tempest.conf'):
+    CONF.set_config_path('./tests/tempest/etc/dev_tempest.conf')
 
 
 class BaseTestCase(base.BaseNetworkTest):
 
-    # This class picks non-admin credentials and run the tempest tests
-
     _lbs_to_delete = []
 
     @classmethod
     def resource_setup(cls):
         super(BaseTestCase, cls).resource_setup()
 
-        cls.listener_protocol = CONF.lbaas.default_listener_protocol
-        cls.pool_protocol = CONF.lbaas.default_pool_protocol
-        cls.hm_protocol = CONF.lbaas.default_health_monitor_protocol
+        credentials = cls.isolated_creds.get_primary_creds()
+        mgr = tempest_clients.Manager(credentials=credentials)
+        auth_provider = mgr.get_auth_provider(credentials)
+        client_args = [auth_provider, 'network', 'regionOne']
 
-        mgr = cls.get_client_manager()
-        auth_provider = mgr.auth_provider
-        client_args = _setup_client_args(auth_provider)
-
-        cls.load_balancers_client = (
-            load_balancers_client.LoadBalancersClientJSON(*client_args))
-        cls.listeners_client = (
-            listeners_client.ListenersClientJSON(*client_args))
+        cls.load_balancers_client = \
+            load_balancers_client.LoadBalancersClientJSON(*client_args)
+        cls.listeners_client = \
+            listeners_client.ListenersClientJSON(*client_args)
         cls.pools_client = pools_client.PoolsClientJSON(*client_args)
         cls.members_client = members_client.MembersClientJSON(*client_args)
-        cls.health_monitors_client = (
-            health_monitors_client.HealthMonitorsClientJSON(*client_args))
+        cls.health_monitors_client = \
+            health_monitors_client.HealthMonitorsClientJSON(*client_args)
 
     @classmethod
     def resource_cleanup(cls):
-
         for lb_id in cls._lbs_to_delete:
             try:
                 lb = cls.load_balancers_client.get_load_balancer_status_tree(
@@ -94,60 +67,26 @@ class BaseTestCase(base.BaseNetworkTest):
                 continue
             for listener in lb.get('listeners'):
                 for pool in listener.get('pools'):
-                    # delete pool's health-monitor
                     hm = pool.get('healthmonitor')
                     if hm:
-                        test_utils.call_and_ignore_notfound_exc(
+                        cls._try_delete_resource(
                             cls.health_monitors_client.delete_health_monitor,
                             pool.get('healthmonitor').get('id'))
                         cls._wait_for_load_balancer_status(lb_id)
-                    test_utils.call_and_ignore_notfound_exc(
-                        cls.pools_client.delete_pool,
-                        pool.get('id'))
+                    cls._try_delete_resource(cls.pools_client.delete_pool,
+                                             pool.get('id'))
                     cls._wait_for_load_balancer_status(lb_id)
-                    # delete pool's members
-                    members = pool.get('members', [])
-                    for member in members:
-                        test_utils.call_and_ignore_notfound_exc(
-                            cls.members_client.delete_member,
-                            pool.get('id'), member.get('id'))
-                        cls._wait_for_load_balancer_status(lb_id)
-                    # delete pool
-                    test_utils.call_and_ignore_notfound_exc(
-                        cls.pools_client.delete_pool, pool.get('id'))
+                    health_monitor = pool.get('healthmonitor')
+                    if health_monitor:
+                        cls._try_delete_resource(
+                            cls.health_monitors_client.delete_health_monitor,
+                            health_monitor.get('id'))
                     cls._wait_for_load_balancer_status(lb_id)
-                # delete listener
-                test_utils.call_and_ignore_notfound_exc(
-                    cls.listeners_client.delete_listener,
-                    listener.get('id'))
+                cls._try_delete_resource(cls.listeners_client.delete_listener,
+                                         listener.get('id'))
                 cls._wait_for_load_balancer_status(lb_id)
-            # delete load-balancer
-            test_utils.call_and_ignore_notfound_exc(
-                cls._delete_load_balancer, lb_id)
-
-        # Wait for all LBs to be fully deleted
-        for lb_id in cls._lbs_to_delete:
-            cls._wait_for_load_balancer_status(
-                lb_id, provisioning_status='DELETED', operating_status=None,
-                delete=True)
-
-        # Wait for straggling port deletes to finish (neutron will sometimes
-        # claim they are deleted before they are totally finished deleting)
-        remaining_ports = cls.ports_client.list_ports(
-            network_id=cls.network['id']).get('ports')
-        tries = 1
-        while remaining_ports and tries <= 10:
-            LOG.warning("Ports still exist on network %s (try #%s): %s",
-                        cls.network['id'], tries, remaining_ports)
-            time.sleep(10)
-            remaining_ports = cls.ports_client.list_ports(
-                network_id=cls.network['id']).get('ports')
-            tries += 1
-
-        if remaining_ports:
-            LOG.error("Failed to remove all remaining ports on network %s. "
-                      "Subnet deletion will probably fail.", cls.network['id'])
-
+            cls._try_delete_resource(
+                cls.load_balancers_client.delete_load_balancer, lb_id)
         super(BaseTestCase, cls).resource_cleanup()
 
     @classmethod
@@ -156,19 +95,21 @@ class BaseTestCase(base.BaseNetworkTest):
         super(BaseTestCase, cls).setUpClass()
 
     def setUp(cls):
-        cls.LOG.info('Starting: {0}'.format(cls._testMethodName))
+        cls.LOG.info(_LI('Starting: {0}').format(cls._testMethodName))
         super(BaseTestCase, cls).setUp()
 
     def tearDown(cls):
         super(BaseTestCase, cls).tearDown()
-        cls.LOG.info('Finished: {0}\n'.format(cls._testMethodName))
+        cls.LOG.info(_LI('Finished: {0}\n').format(cls._testMethodName))
 
     @classmethod
     def _create_load_balancer(cls, wait=True, **lb_kwargs):
-        lb = cls.load_balancers_client.create_load_balancer(**lb_kwargs)
-        if wait:
-            cls._wait_for_load_balancer_status(lb.get('id'))
-
+        try:
+            lb = cls.load_balancers_client.create_load_balancer(**lb_kwargs)
+            if wait:
+                cls._wait_for_load_balancer_status(lb.get('id'))
+        except Exception:
+            raise Exception(_("Failed to create load balancer..."))
         cls._lbs_to_delete.append(lb.get('id'))
         return lb
 
@@ -199,14 +140,10 @@ class BaseTestCase(base.BaseNetworkTest):
                                        provisioning_status='ACTIVE',
                                        operating_status='ONLINE',
                                        delete=False):
-        interval_time = 1
-        timeout = 600
+        interval_time = 10
+        timeout = 300
         end_time = time.time() + timeout
         lb = {}
-        # When running with no-op drivers there is no actual health to
-        # observe, so disable operating_status checks when running no-op.
-        if CONF.lbaas.test_with_noop:
-            operating_status = None
         while time.time() < end_time:
             try:
                 lb = cls.load_balancers_client.get_load_balancer(
@@ -220,60 +157,29 @@ class BaseTestCase(base.BaseNetworkTest):
                             _("loadbalancer {lb_id} not"
                               " found").format(
                                   lb_id=load_balancer_id))
-                if lb.get('provisioning_status') == provisioning_status:
-                    if operating_status is None:
-                        break
-                    elif lb.get('operating_status') == operating_status:
-                        break
+                if (lb.get('provisioning_status') == provisioning_status and
+                        lb.get('operating_status') == operating_status):
+                    break
                 time.sleep(interval_time)
-            except exceptions.NotFound:
+            except exceptions.NotFound as e:
                 # if wait is for delete operation do break
                 if delete:
                     break
                 else:
                     # raise original exception
-                    raise
+                    raise e
         else:
-            if delete:
-                raise exceptions.TimeoutException(
-                    _("Waited for load balancer {lb_id} to be deleted for "
-                      "{timeout} seconds but can still observe that it "
-                      "exists.").format(
-                          lb_id=load_balancer_id,
-                          timeout=timeout))
-            else:
-                raise exceptions.TimeoutException(
-                    _("Wait for load balancer ran for {timeout} seconds and "
-                      "did not observe {lb_id} reach {provisioning_status} "
-                      "provisioning status and {operating_status} "
-                      "operating status.").format(
-                          timeout=timeout,
-                          lb_id=load_balancer_id,
-                          provisioning_status=provisioning_status,
-                          operating_status=operating_status))
+            raise Exception(
+                _("Wait for load balancer ran for {timeout} seconds and did "
+                  "not observe {lb_id} reach {provisioning_status} "
+                  "provisioning status and {operating_status} "
+                  "operating status.").format(
+                      timeout=timeout,
+                      lb_id=load_balancer_id,
+                      provisioning_status=provisioning_status,
+                      operating_status=operating_status))
         return lb
 
-    @classmethod
-    def _wait_for_neutron_port_delete(cls, port_id):
-        """
-        Neutron port deletion is asynchronous. This method waits, up to a
-        timeout for the port deletion to complete.
-        """
-        interval_time = 1
-        timeout = 600
-        end_time = time.time() + timeout
-        while time.time() < end_time:
-            try:
-                cls.ports_client.show_port(port_id)
-                time.sleep(interval_time)
-            except exceptions.NotFound:
-                break
-        else:
-            raise exceptions.TimeoutException(
-                    _("Waited for port {port_id} to be deleted for "
-                      "{timeout} seconds but can still observe that it "
-                      "exists.").format(port_id=port_id, timeout=timeout))
-
     @classmethod
     def _create_listener(cls, wait=True, **listener_kwargs):
         listener = cls.listeners_client.create_listener(**listener_kwargs)
@@ -317,30 +223,28 @@ class BaseTestCase(base.BaseNetworkTest):
                 cls.load_balancer.get('id'))
         return pool
 
-    def _create_health_monitor(self, wait=True, cleanup=True,
-                               **health_monitor_kwargs):
-        hm = self.health_monitors_client.create_health_monitor(
+    @classmethod
+    def _create_health_monitor(cls, wait=True, **health_monitor_kwargs):
+        hm = cls.health_monitors_client.create_health_monitor(
             **health_monitor_kwargs)
-        if cleanup:
-            self.addCleanup(self._delete_health_monitor, hm.get('id'))
         if wait:
-            self._wait_for_load_balancer_status(self.load_balancer.get('id'))
+            cls._wait_for_load_balancer_status(cls.load_balancer.get('id'))
         return hm
 
-    def _delete_health_monitor(self, health_monitor_id, wait=True):
-        self.health_monitors_client.delete_health_monitor(health_monitor_id)
+    @classmethod
+    def _delete_health_monitor(cls, health_monitor_id, wait=True):
+        cls.health_monitors_client.delete_health_monitor(health_monitor_id)
         if wait:
-            self._wait_for_load_balancer_status(self.load_balancer.get('id'))
+            cls._wait_for_load_balancer_status(cls.load_balancer.get('id'))
 
-    def _update_health_monitor(self, health_monitor_id, wait=True,
+    @classmethod
+    def _update_health_monitor(cls, health_monitor_id, wait=True,
                                **health_monitor_kwargs):
-        health_monitor = self.health_monitors_client.update_health_monitor(
+        health_monitor = cls.health_monitors_client.update_health_monitor(
             health_monitor_id, **health_monitor_kwargs)
         if wait:
-            self._wait_for_load_balancer_status(
-                self.load_balancer.get('id'))
-            health_monitor = self.health_monitors_client.get_health_monitor(
-                health_monitor_id)
+            cls._wait_for_load_balancer_status(
+                cls.load_balancer.get('id'))
         return health_monitor
 
     @classmethod
@@ -373,12 +277,7 @@ class BaseTestCase(base.BaseNetworkTest):
         statuses = cls.load_balancers_client.get_load_balancer_status_tree(
             load_balancer_id=load_balancer_id)
         load_balancer = statuses['loadbalancer']
-
-        # When running with no-op drivers there is no actual health to
-        # observe, so disable operating_status checks when running no-op.
-        if not CONF.lbaas.test_with_noop:
-            assert 'ONLINE' == load_balancer['operating_status']
-
+        assert 'ONLINE' == load_balancer['operating_status']
         assert 'ACTIVE' == load_balancer['provisioning_status']
 
         if listener_ids:
@@ -400,16 +299,13 @@ class BaseTestCase(base.BaseNetworkTest):
     @classmethod
     def _check_status_tree_thing(cls, actual_thing_ids, status_tree_things):
         found_things = 0
+        status_tree_things = status_tree_things
         assert len(actual_thing_ids) == len(status_tree_things)
         for actual_thing_id in actual_thing_ids:
             for status_tree_thing in status_tree_things:
                 if status_tree_thing['id'] == actual_thing_id:
-                    # When running with no-op drivers there is no actual
-                    # health to observe, so disable operating_status checks
-                    # when running no-op.
-                    if not CONF.lbaas.test_with_noop:
-                        assert 'ONLINE' == (
-                            status_tree_thing['operating_status'])
+                    assert 'ONLINE' == (
+                        status_tree_thing['operating_status'])
                     assert 'ACTIVE' == (
                         status_tree_thing['provisioning_status'])
                     found_things += 1
@@ -422,43 +318,3 @@ class BaseTestCase(base.BaseNetworkTest):
             case_name=cls.__name__
         )
         return name
-
-    def _test_provisioning_status_if_exists(self, created_obj, get_obj):
-        prov_status_old = created_obj.pop('provisioning_status', None)
-        prov_status_new = get_obj.pop('provisioning_status', None)
-
-        if prov_status_old:
-            self.assertIn('PENDING_', prov_status_old)
-        if prov_status_new:
-            self.assertEqual(n_constants.ACTIVE, prov_status_new)
-        if not created_obj.get('updated_at') and get_obj.get('updated_at'):
-            get_obj['updated_at'] = None
-
-
-class BaseAdminTestCase(BaseTestCase):
-
-    # This class picks admin credentials and run the tempest tests
-
-    @classmethod
-    def resource_setup(cls):
-
-        super(BaseAdminTestCase, cls).resource_setup()
-
-        mgr = cls.get_client_manager(credential_type='admin')
-        auth_provider_admin = mgr.auth_provider
-        client_args = _setup_client_args(auth_provider_admin)
-
-        cls.load_balancers_client = (
-            load_balancers_client.LoadBalancersClientJSON(*client_args))
-        cls.listeners_client = (
-            listeners_client.ListenersClientJSON(*client_args))
-        cls.pools_client = (
-            pools_client.PoolsClientJSON(*client_args))
-        cls.members_client = (
-            members_client.MembersClientJSON(*client_args))
-        cls.health_monitors_client = (
-            health_monitors_client.HealthMonitorsClientJSON(*client_args))
-
-    @classmethod
-    def resource_cleanup(cls):
-        super(BaseAdminTestCase, cls).resource_cleanup()
